@@ -1,10 +1,10 @@
 package Class::Factory;
 
-# $Id: Factory.pm 40 2006-08-02 05:51:40Z cwinters $
+# $Id: Factory.pm 46 2007-11-07 00:06:38Z cwinters $
 
 use strict;
 
-$Class::Factory::VERSION = '1.05';
+$Class::Factory::VERSION = '1.06';
 
 my %CLASS_BY_FACTORY_AND_TYPE  = ();
 my %FACTORY_INFO_BY_CLASS      = ();
@@ -123,6 +123,48 @@ sub register_factory_type {
 }
 
 
+sub remove_factory_type {
+    my ( $item, @object_types ) = @_;
+    my $class = ref $item || $item;
+
+    for my $object_type (@object_types) {
+        unless ( $object_type )  {
+            $item->factory_error(
+                "Cannot remove factory type from '$class': no type defined"
+            );
+        }
+
+        delete $CLASS_BY_FACTORY_AND_TYPE{ $class }->{ $object_type };
+    }
+}
+
+
+sub unregister_factory_type {
+    my ( $item, @object_types ) = @_;
+    my $class = ref $item || $item;
+
+    for my $object_type (@object_types) {
+        unless ( $object_type )  {
+            $item->factory_error(
+                "Cannot remove factory type from '$class': no type defined"
+            );
+        }
+
+        delete $REGISTER{ $class }->{ $object_type };
+
+        # Also delete from $CLASS_BY_FACTORY_AND_TYPE because if the object
+        # type has already been instantiated, then it will have been processed
+        # by add_factory_type(), thus creating an entry in
+        # $CLASS_BY_FACTORY_AND_TYPE. We can call register_factory_type()
+        # again, but when we try to instantiate an object via
+        # get_factory_class(), it will find the old entry in
+        # $CLASS_BY_FACTORY_AND_TYPE and use that.
+
+        delete $CLASS_BY_FACTORY_AND_TYPE{ $class }->{ $object_type };
+    }
+}
+
+
 sub get_loaded_classes {
     my ( $item ) = @_;
     my $class = ref $item || $item;
@@ -180,6 +222,14 @@ sub get_my_factory {
 
 sub get_my_factory_type {
     my ( $item ) = @_;
+    $item->get_factory_type_for($item);
+}
+
+
+# Return the type that the factory uses to create a given object or class.
+
+sub get_factory_type_for {
+    my ( $self, $item ) = @_;
     my $impl_class = ref( $item ) || $item;
     my $impl_info = $FACTORY_INFO_BY_CLASS{ $impl_class };
     if ( ref( $impl_info ) eq 'ARRAY' ) {
@@ -246,14 +296,22 @@ Class::Factory - Base class for dynamic factory classes
   
   my $registered_class = My::Factory->get_registered_class( 'type' );
 
- # Ask the object created by the factory: Where did I come from?
+  # Ask the object created by the factory: Where did I come from?
  
- my $custom_object = My::Factory->new( 'custom' );
- print "Object was created by factory: ",
+  my $custom_object = My::Factory->new( 'custom' );
+  print "Object was created by factory: ",
        $custom_object->get_my_factory, " ",
        "and is of type: ",
        $custom_object->get_my_factory_type;
 
+  # Remove a factory type
+
+  My::Factory->remove_factory_type('perl');
+
+  # Unregister a factory type
+
+  My::Factory->unregister_factory_type('blech');
+
 =head1 DESCRIPTION
 
 This is a simple module that factory classes can use to generate new
@@ -601,6 +659,29 @@ parameters are not given then we call C<factory_error()>. A
 C<factory_log()> message is issued if the type has already been
 registered.
 
+B<remove_factory_type( @object_types )>
+
+Removes a factory type from the factory. This is the opposite of
+C<add_factory_type()>. No return value.
+
+Removing a factory type is useful if a subclass of the factory wants to
+redefine the mapping for the factory type. C<add_factory_type()> doesn't allow
+overriding a factory type, so you have to remove it first.
+
+B<unregister_factory_type( @object_types )>
+
+Unregisters a factory type from the factory. This is the opposite of
+C<register_factory_type()>. No return value.
+
+Unregistering a factory type is useful if a subclass of the factory wants to
+redefine the mapping for the factory type. C<register_factory_type()> doesn't
+allow overriding a factory type, so you have to unregister it first.
+
+B<get_factory_type_for( $class )>
+
+Takes an object or a class name string and returns the factory type that is
+used to construct that class. Returns undef if there is no such factory type.
+
 B<get_loaded_classes()>
 
 Returns a sorted list of the currently loaded classes. If no classes
@@ -688,7 +769,7 @@ which would print:
 
 =head1 COPYRIGHT
 
-Copyright (c) 2002-2006 Chris Winters. All rights reserved.
+Copyright (c) 2002-2007 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -701,6 +782,8 @@ John Vlissides. Addison Wesley Longman, 1995. Specifically, the
 
 =head1 AUTHORS
 
+Fred Moyer <fred@redhotpenguin.com> is the current maintainer.
+
 Chris Winters E<lt>chris@cwinters.comE<gt>
 
 Eric Andreychek E<lt>eric@openthought.netE<gt> implemented overridable
@@ -711,3 +794,7 @@ for 'get_my_factory()' and 'get_my_factory_type()'
 
 Sebastian Knapp <giftnuss@netscape.net> contributed the idea for
 'get_registered_class()'
+
+Marcel Gruenauer <marcel@cpan.org> contributed the methods
+remove_factory_type() and unregister_factory_type().
+
